/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
*
*  MODULE :             List Management
*  SCOPE :              Implements list management
*  FILE :               listtempl.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) Eric Abecassis, 1994
*  HISTORY :
*
******************************************************************************/

/* IMPORTS *******************************************************************/

#ifdef __GNUG__
#pragma interface
#endif

#ifndef __string_H
#define __string_H
#include <string.h>
#endif

template <class Elem> class ListVal ;
template <class Elem> class ListPtr ;


// Iterator for a list of values.
template <class Elem> class IterListVal : public IterList {
  public :
  
  IterListVal(ListVal<Elem>& l, int b=0, int e=-1) : IterList(l, b,e) {};

  Elem& get() { return *(Elem*)this->IterList::get() ; }

  void remove() { delete (Elem*)this->IterList::remove() ; }

  void insert(Elem& el) { this->IterList::insert(new Elem(el)) ; }

  void insertRef(Elem* el) { this->IterList::insert(el) ; }
} ;

typedef int __YOOC_LIST_CTOR ;

extern const __YOOC_LIST_CTOR YOOC_LIST_CTOR ;

// Template for a list of values
template <class Elem> class ListVal : public List {
  public :
  
  ListVal() {};

  ListVal(ListVal<Elem>& l, __YOOC_LIST_CTOR) {
    this->capture(l) ;
  }
  
  ListVal(const ListVal<Elem>& l) {
    IterListVal<Elem> iterDst(*this) ;
    IterListVal<Elem> iterSrc(*(ListVal<Elem>*)&l) ;
    for (; iterSrc.more(); iterSrc.next()) {
      iterDst.insert(iterSrc.get()) ;
    }
  }
  
  ~ListVal() { this->clear() ; } ;
  
  void insertVal(const Elem& elem, int i) { 
    this->List::insert(new Elem(elem), i) ; 
  }

  // Caller is responsable to allocate the element before insertion.
  // Deletion of the element will be automatic when the list will be deleted
  void insertRef(Elem* elem, int i) { 
    this->List::insert(elem, i) ; 
  }

  void insert(const Elem& elem, int i) { 
    this->List::insert(new Elem(elem), i) ; 
  }

  void remove(int i) { delete (Elem*)this->List::remove(i) ; }

  Elem& get(int i) { return *(Elem*)this->List::get(i)->elem ; } 

  Elem& operator[](int i) { return get(i) ; }

  ListVal<Elem>& operator+=(const Elem& el) {
    this->insert(el, 0) ; return *this ;
  }

  void clear() { while (size) { delete (Elem*)this->List::remove(0) ; } } ;
} ;


// Iterator for a list of references
template <class Elem> class IterListPtr : public IterList {
  public :
  
  IterListPtr(ListPtr<Elem>& l, int b=0, int e=-1) : IterList(l, b,e) {};

  Elem* get() { return (Elem*)this->IterList::get() ; }

  Elem* remove() { return (Elem*)this->IterList::remove() ; }

  void insert(Elem* el) { this->IterList::insert(el) ; }
} ;


// Template for a list of references
template <class Elem> class ListPtr : public List {
  public :

  ListPtr() {} ;

  ListPtr(ListPtr<Elem>& l, __YOOC_LIST_CTOR) {
    this->capture(l) ;
  }

  ListPtr(const ListPtr<Elem>& l) {
    IterListPtr<Elem> iterDst(*this) ;
    IterListPtr<Elem> iterSrc(*(ListPtr<Elem>*)&l) ;
    for (; iterSrc.more(); iterSrc.next()) {
      iterDst.insert(iterSrc.get()) ;
    }
  }
  
  ~ListPtr() { this->clear() ; }
  
  void insert(Elem* const elem, int i) {
    this->List::insert((void*)elem, i) ;
  }

  Elem* remove(int i) { return (Elem*)this->List::remove(i) ; }

  Elem* get(int i) { return (Elem*)this->List::get(i)->elem ; } 

  Elem* operator[](int i) { return get(i) ; }

  ListPtr<Elem>& operator+=(Elem* const el) {
    this->insert(el, 0) ; return *this ;
  }

  void clear() { while (this->size != 0) { this->List::remove(0) ; } }
} ;


class SvcSocket ;

template <class Elem> 
SvcSocket& operator<<(SvcSocket& s, const ListVal<Elem>& l) 
{
  s << l.count() ;
  IterListVal<Elem> iter(*(ListVal<Elem>*)&l) ;
  for (; iter.more(); iter.next()) {
    s << iter.get() ;
    if (s.error()) return s;
  }
  return s ;
} 


template <class Elem> 
SvcSocket& operator>>(SvcSocket& s, ListVal<Elem>& l) 
{ 
  int size ;
  s >> size ;
  IterListVal<Elem> iter(l) ;
  for (int i = 0; i < size; i++) {    
    Elem* el = new Elem ;
    s >> *el ;
    if (s.error()) return s;
    iter.insertRef(el) ;
  } 
  return s ;  
}




